#!%%PMXPREFIX%%/bin/perl -w

use strict;
use Getopt::Long qw(GetOptions);
use IO::Select;
use ActiveState::File::Atomic;
use ActiveState::Handy qw(run);
use ActiveState::Config qw(:path);
use PureMessage::Config qw(%config);
use ActiveState::File::Atomic;
use PureMessage::Service;

my $enable_flag=0;
my $disable_flag=0;
my $status_flag=0;
my $passthru_flag=0; 
my $tempfail_flag=0;
my $status;

GetOptions('enable'   => \$enable_flag,
           'disable'  => \$disable_flag,
           'status'   => \$status_flag,
           'passthru' => \$passthru_flag,
           'tempfail' => \$tempfail_flag) or die(usage());

if (($disable_flag && $enable_flag) || 
    ($passthru_flag && $tempfail_flag)) {
    die(usage());
}

if ($status_flag) {
    $status = mta_ipblock_status();
    if($status) {
        print "Enabled\n";
    }
    else {
        print "Disabled\n";
    }
}
elsif ($disable_flag || $enable_flag) {
    my $service_port = _get_port(); 

    die("Unable to determine blocklist port spec") if (!$service_port);

    my %opts = ( 'SET'      => ($enable_flag)   ? 1 : 0,
                 'PASSTHRU' => ($tempfail_flag) ? 0 : 1,
                 'PORT'     =>  $service_port );
    $status = mta_ipblock_status(%opts);
}
else {
    die(usage());
}

exit($status);

sub mta_ipblock_status {
    my %opts = @_;
    my $func;
    my $status;
    my %mta_mapping = (
	sendmail => \&_sendmail,
	postfix  => \&_postfix,
	jsms     => \&_jsms,
    );

    my $mta = _mta_type();
    if (!($func = $mta_mapping{$mta})) {
	warn "MTA unsupported\n";
        $status = 2;
    }
    else {
        if ( keys %opts ) {
	    my ($retval,$modified) = $func->(%opts);
	    $status = ($retval eq $opts{'SET'}) ? 0 : 1;
            if($modified) {
                print STDERR <<'EOT'
You must restart the MTA for your changes to have effect
EOT
            }
	}
	else {
	    ($status,undef) = $func->();
	}
    }
    return ($status);
}

sub _sendmail {
    my %opts = @_;
    my $path = $config{'prefix'}.'/sendmail/etc/mail/sendmail.mc';
    my $writable = 0;
    my $at;
    my $retval = 2;
    my $modified = 0;
    my ($match,$on,$options);

    $writable = 1 if @_;
    eval{$at = ActiveState::File::Atomic->new($path, writable => $writable)};
    return($retval) if ($@);

    my $contents = $at->slurp;
    if($contents =~ /^((dnl\s)?(FEATURE\(`sockmap',`(.*?)',(.*?),(.*?)\)))/m) {
#       $3 = feature without optional DNL prefix, $4 = port. Not used currently
        ($match,$on,$options) = ($1,!defined $2,$5);
    }
    else {
        ($on,$options) = (0,'');
    }

    if(@_) {
	my $status=$opts{'SET'};
	my $replace;

        $replace  = ($status) ? "" : "dnl ";
        $replace .= "FEATURE(`sockmap',";
        $replace .= "`".$opts{'PORT'}."',";
        $replace .= $options.",";
        $replace .= ($opts{'PASSTHRU'}) ? "" : "`t'";
        $replace .= ")";

        if($match) { 
	    if($replace ne $match) {
	        $contents =~ s/\Q$match\E/$replace/;
                $modified = 1;
            }
	}
        else { # Not in the config, add it before the first FEATURE line found.
            $contents =~ s/^(FEATURE.*)$/$replace\n$1/m;
            $modified = 1;
        }

        if ($modified) {
	    $at->commit_string($contents);
        }
	$retval=$status;
    }
    else {
	$retval=$on;
    }
    return ($retval,$modified);
}

# Modification via pmxbl.conf file of PureMessage-JSMS 
sub _jsms {
    my %opts = @_;
    my $enabled = "pmxbl#options.enabled";
    my $writable = 0;
    my $retval;
    my $p;
    my $modified = 0;
    
    $p = ActiveState::Config->new();
    $writable = 1 if @_; 
    $p->parse_file($config{'prefix'}."/jsms/etc/pmxbl.conf", 
                   writable => $writable);
    $retval = path_value($p->root, $enabled);

    if (@_) {
        my $sspec   = "pmxbl#options.socketspec";
        my $fmode   = "pmxbl#options.onerror_passthru";
	my $status  = $opts{'SET'};
	if ( (!defined($retval)) || ($retval != $status) ) {
	    path_value($p->root, $enabled, $status);
            $modified = 1;
	}
        my $port = path_value($p->root, $sspec);
        if ($opts{'PORT'} ne $port) {
	    path_value($p->root, $sspec, $opts{'PORT'});
            $modified = 1;
        }
        my $passthru = path_value($p->root, $fmode) || 0;
        if ($opts{'PASSTHRU'} != $passthru) {
	    path_value($p->root, $fmode, $opts{'PASSTHRU'});
            $modified = 1;
        }
        if($modified) {
            $p->save;
        }
        $retval = $status;
    }
    else {
	if(!defined($retval)) {
	    $retval = 0; # No value = off
	}
    }
    return($retval,$modified);
}

# Modification via postconf utility of postfix 
sub _postfix {
    my %opts = @_;
    my $attr='smtpd_client_restrictions';
    my $retval = 2;
    my $value;
    my $modified = 0;

    eval {$value=_run_postconf(ARGS=>"-h $attr",READ=>1)};
    unless($@) {
        chomp($value);
        $retval=($value) ? 1 : 0;
        if (@_) {
            my $status = $opts{'SET'};
            my $port = _conv_port($opts{'PORT'});
            my $replace = "";
            if($status) {
                if ($opts{'PASSTHRU'}) {
                    $replace = "ignore_policy_error,"
                }
                $replace .= "check_policy_service $port";
            }

            if($replace ne $value) {
                eval{
                    _run_postconf(ARGS=>"-e '$attr=$replace'");
                };
                unless($@) {
                    $retval = $status;
                    $modified = 1;
                }
                else {
                    $retval = 2;
                }
            }
        } 
    }
    return($retval,$modified);
}

# Postfix has a slightly different port spec to the 'sendmail style' one 
# so convert it.
sub _conv_port {
    my $sspec = shift;

    my ($proto,$value)=split(":",$sspec);
    if($value =~ /\@/) {
        my @specarr=split("@",$value);
        $sspec=$proto.':'.$specarr[1].':'.$specarr[0];
    }
    return ($sspec);
}

sub _mta_type {
   my $mta;

   my @services = PureMessage::Service->Available;
   foreach my $s (@services) {
       if ($s->type eq 'smtp') {
           $mta=lc($s->name);
           last;
       }
   }

   if (!$mta) {
       # Because JSMS has no 'service' support we check for the existence of
       # the pmx jsms directory to ascertain whether its installed.
       $mta='jsms' if (-d $config{'prefix'}.'/jsms');
   }
   # The return value is used as a key, so we dont return 'undef'.
   return (($mta) ? $mta : '');
}

sub _get_port {
    if($config{'blocklist'} && $config{'blocklist'}{'Policy'}) {
        return($config{'blocklist'}{'Policy'}{'port'});
    }
    return undef;
}

sub _run_postconf {
   my %opts=(
      READ=>0,
      ARGS=>'',
      @_,
   );
   my $cmd=$config{'prefix'}.'/postfix/sbin/postconf';
   my $res;

   if ($opts{READ}) {
       open (FD, $cmd." $opts{ARGS} 2>/dev/null |") or die("Cant run $cmd $!");
       $res = <FD>;
       close(FD) or die("Cant run $cmd $?");
   }
   else {
       # Will 'die' on error
       $res=run("\@$cmd $opts{ARGS} 2>/dev/null >/dev/null");
   }
   return($res);
}

sub usage { die <<USAGE }
Usage: pmx-blocklist [options] 
Enables or disables IP blocking for your MTA.

Options:
   --status
   --enable [--passthru] [--tempfail]
   --disable 

Examples:

 # Display the current status of IP blocking
 pmx-blocklist --status

 # Enable IP blocking (tempfail on error)
 pmx-blocklist --enable --tempfail

 # Enable IP blocking (passthru on error - default)
 pmx-blocklist --enable --passthru

 # Disable IP blocking
 pmx-blocklist --disable

USAGE

__END__

=head1 NAME

pmx-blocklist - Enables or disables IP blocking for a PureMessage MTA.

=head1 SYNOPSIS

pmx-blocklist [I<options>]

Options:
   --status
   --enable [--passthru] [--tempfail]
   --disable

=head1 DESCRIPTION

The C<pmx-blocklist> program is used to enable or disable IP blocking,
configure the default failure behavior in MTAs installed by PureMessage
(sendmail or Postfix), and reload configuration options from
F</opt/pmx/etc/pmx.d/blocklist.conf>. The default behavior is to pass through on error (see
the third example below).

=head1 EXAMPLES

To display the current status of IP blocking:

   pmx-blocklist --status

To enable IP blocking (tempfail on error):

   pmx-blocklist --enable --tempfail

To enable IP blocking (pass through on error - default):

   pmx-blocklist --enable --passthru

To disable IP blocking:

   pmx-blocklist --disable

To reload I<etc/pmx.d/blocklist.conf>:

   pmx-blocklist --enable

=head1 COPYRIGHT

Copyright (C) 2000-2008 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
